/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
* @brief Example template project.
* @defgroup nrf_templates_example Example Template
*
*/

#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include "nrf_gpio.h"
#include "nrf_drv_twi.h"
#include "bsp.h"
#include "app_error.h"
#include "nrf_drv_config.h"
#include "app_util_platform.h"
#include "nrf_delay.h"
#include "app_uart.h"

#include "mpu9150_register_map.h"
#include "mpu9150.h"

//#include "Segger_RTT.h"

#define UART_TX_BUF_SIZE 256 /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE 1   /**< UART RX buffer size. */


int16_t adc_result;

/**
 * @brief UART events handler.
 */
void uart_events_handler(app_uart_evt_t * p_event)
{
}

/**
 * @brief UART initialization.
 */
void uart_config(void)
{
    uint32_t                     err_code;
    const app_uart_comm_params_t comm_params =
    {
        RX_PIN_NUMBER,
        TX_PIN_NUMBER,
        RTS_PIN_NUMBER,
        CTS_PIN_NUMBER,
        APP_UART_FLOW_CONTROL_DISABLED,
        false,
        UART_BAUDRATE_BAUDRATE_Baud38400
    };
    
    APP_UART_FIFO_INIT(&comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_events_handler,
                       APP_IRQ_PRIORITY_LOW,
                       err_code);

    APP_ERROR_CHECK(err_code);
}

// TWI event handler. Processing TWI events originating from
// nrf_drv_twi_tx()
static void twi_event_handler(nrf_drv_twi_evt_t *evt)
{
    mpu9150_on_twi_event(evt);
}

static void gpio_init()
{
    nrf_gpio_range_cfg_output(LED_1, LED_4);
    nrf_gpio_pin_set(LED_3);
    nrf_gpio_pin_set(LED_2);
    nrf_gpio_pin_set(LED_1);
    nrf_gpio_pin_set(LED_4);
}


/**
 * @brief Function for application main entry.
 */
int main(void)
{
    uint32_t err_code;
    ret_code_t ret_code; 
    
    uart_config();
    printf("\033[2J\033[;HStart. Compiled @ %s\n\r", __TIME__);
    
    gpio_init(); // Initated LEDS for nRF51 DK
    
    // Initate and configure TWI instance
    const nrf_drv_twi_t     p_twi_instance = NRF_DRV_TWI_INSTANCE(1); // Set up TWI instance 1 with default values
    nrf_drv_twi_config_t    p_twi_config = NRF_DRV_TWI_DEFAULT_CONFIG(1);// Set up TWI configuration default values
    p_twi_config.scl = 1; // Modify pins to our needs
    p_twi_config.sda = 2;
    
    // Initate TWI module using event handler. 
    ret_code = nrf_drv_twi_init(&p_twi_instance, &p_twi_config, twi_event_handler); // Initiate twi driver with instance and configuration values
    APP_ERROR_CHECK(ret_code); // Check for errors in return value
    nrf_drv_twi_enable(&p_twi_instance); // Enable the TWI instance


    // Initate MPU using specified TWI instance
    err_code = mpu9150_init(&p_twi_instance);
    APP_ERROR_CHECK(err_code);
    
    // Configure MPU with default values
    mpu9150_config_t config = MPU9150_DEFAULT_CONFIG();
    err_code = mpu9150_config(&config);
    APP_ERROR_CHECK(err_code);
    
    uint16_t accel_range;
    mpu9150_accel_sensitivity_get(&accel_range);
    printf("\r\nAccelerometer range: %d\n\r", accel_range);
    
    // Variables to hold accel, gyro and temperature values. 
    accel_values_t  accel_values;
    gyro_values_t   gyro_values;
    temp_value_t    temperature;
    magn_values_t   magn_values;
    
    nrf_delay_ms(3000);
    
    while(1)
    {
        mpu9150_read_accel(&accel_values);
        printf("\033[2J\033[;HAccelerometer:\n\rX %8.4f\n\rY %8.4f\n\rZ %8.4f\n\r", (float)accel_values.x/accel_range, (float)accel_values.y/accel_range,
            (float)accel_values.z/accel_range);
        
        mpu9150_read_gyro(&gyro_values);
        printf("\n\rGyro:\n\rX %05d\n\rY %05d\n\rZ %05d\n\r", gyro_values.x, gyro_values.y, gyro_values.z);
        
        
        mpu9150_read_magn(&magn_values);
        printf("\n\rMagnetometer:\n\rX %06d\n\rY %06d\n\rZ %06d\n\r", magn_values.x, magn_values.y,
            magn_values.z);
        
        mpu9150_read_temp(&temperature);
        printf("\n\rTemperature: %4.2f%cC\n\r", (((float)temperature/340)+(float)36), 248);
        
        nrf_delay_ms(1000);
        nrf_gpio_pin_toggle(LED_4);
    }
}
/** @} */
