/*
 * 
 */

#include <stdbool.h>
#include <stdint.h>

#include "nrf_drv_twi.h"
#include "mpu9150.h"
#include "mpu9150_register_map.h"
#include "Segger_RTT.h"
#include "app_error.h"
#include "nrf_delay.h"

static nrf_drv_twi_t const * m_twi_instance;
static bool twi_tx_done = false;
static bool twi_rx_done = false;


void mpu9150_on_twi_event(nrf_drv_twi_evt_t *evt)
{ 
    switch(evt->type)
    {
        case NRF_DRV_TWI_TX_DONE:
            twi_tx_done = true;
            break;
        case NRF_DRV_TWI_RX_DONE:
            twi_rx_done = true;
            break;
        case NRF_DRV_TWI_ERROR:
            switch(evt->error_src)
            {
                case NRF_TWI_ERROR_ADDRESS_NACK:
                    SEGGER_RTT_printf(0, "TWI ERR: NACK rxed after sending the addr\n");
                    break;
                case NRF_TWI_ERROR_OVERRUN_NACK:
                    SEGGER_RTT_printf(0, "Byte rxed in RXD register before read of the last rxed byte (data loss)\n");
                    break;
                case NRF_TWI_ERROR_DATA_NACK:
                    SEGGER_RTT_printf(0, "NACK rxed after sending a byte\n");
                    break;
                default:
                    break;
            }
            break;
        default:
            SEGGER_RTT_printf(0, "Undefined TWI evt\n");
            break;
    }        
}

// Write two or more bytes to consecutive registers 
static uint32_t twi_write_burst(uint8_t reg, uint8_t const * p_data, uint32_t length)
{
    uint32_t err_code;
    uint32_t timeout = MPU9150_ERR_TWI_TIMEOUT;

    err_code = nrf_drv_twi_tx(m_twi_instance, MPU9150_ADDRESS, &reg, 1, true);
    APP_ERROR_CHECK(err_code);
    
    while((!twi_tx_done) && timeout--);  
    if(!timeout) return MPU9150_ERR_TWI_TIMEOUT;
    twi_tx_done = false;
    
    err_code = nrf_drv_twi_tx(m_twi_instance, MPU9150_ADDRESS, p_data, length, false);
    APP_ERROR_CHECK(err_code);
    
    while((!twi_tx_done) && timeout--);
    if(!timeout) return MPU9150_ERR_TWI_TIMEOUT;
    twi_tx_done = false;
    
    return err_code;
}

// Write a single byte to register 
static uint32_t twi_write_single(uint8_t reg, uint8_t const data)
{
    uint32_t err_code;
    uint32_t timeout = MPU9150_ERR_TWI_TIMEOUT;
    
    uint8_t packet[2];
    packet[0] = reg;
    packet[1] = data;
    
    err_code = nrf_drv_twi_tx(m_twi_instance, MPU9150_ADDRESS, packet, 2, false);
    APP_ERROR_CHECK(err_code);
    
    while((!twi_tx_done) && timeout--);
    if(!timeout) return MPU9150_ERR_TWI_TIMEOUT;
    
    twi_tx_done = false;
    
    return err_code;
}

// Read two or more bytes from consecutive registers 
static uint32_t twi_read_burst(uint8_t reg, uint8_t * p_data, uint32_t length)
{
    uint32_t err_code;
    uint32_t timeout = MPU9150_ERR_TWI_TIMEOUT;
    
    err_code = nrf_drv_twi_tx(m_twi_instance, MPU9150_ADDRESS, &reg, 1, false);
    APP_ERROR_CHECK(err_code);
    
    while((!twi_tx_done) && timeout--);  
    if(!timeout) return MPU9150_ERR_TWI_TIMEOUT;
    twi_tx_done = false;
    
    err_code = nrf_drv_twi_rx(m_twi_instance, MPU9150_ADDRESS, p_data, length, false);
    APP_ERROR_CHECK(err_code);
    
    while((!twi_rx_done) && timeout--);
    if(!timeout) return MPU9150_ERR_TWI_TIMEOUT;
    twi_rx_done = false;
    
    return err_code;
}

// Just a wrapper function. Probably unnecessary
uint32_t mpu9150_write_registers(uint8_t reg, uint8_t const * p_data, uint32_t length)
{
    return twi_write_burst(reg, p_data, length);
}

// Just a wrapper function. Probably unnecessary
uint32_t mpu9150_read_registers(uint8_t reg, uint8_t * p_data, uint32_t length)
{
    return twi_read_burst(reg, p_data, length);
}

uint32_t mpu9150_config(mpu9150_config_t * config)
{
    uint32_t err_code;
    uint8_t *data; 
    data = (uint8_t*)config;
    
    err_code = twi_write_burst(MPU9150_REG_SMPLRT_DIV, data, 4);
    APP_ERROR_CHECK(err_code);
    
    return err_code;
}

uint32_t mpu9150_init(nrf_drv_twi_t const * const p_instance)
{
    uint32_t err_code;
    
    m_twi_instance = p_instance;
    
    uint8_t reset_value = 0x04U | 0x02U | 0x01U; // Resets gyro, accelerometer and temperature sensor signal paths.
    err_code = twi_write_single(MPU9150_REG_SIGNAL_PATH_RESET, reset_value);
    APP_ERROR_CHECK(err_code);
    
    err_code = twi_write_single(MPU9150_REG_PWR_MGMT_1, 1); // Start up MPU
    APP_ERROR_CHECK(err_code);
    
    return err_code;
}


uint32_t mpu9150_read_accel(accel_values_t * accel_values)
{
    uint32_t err_code;
    uint8_t raw_values[6];
    err_code = twi_read_burst(MPU9150_REG_ACCEL_XOUT_H, raw_values, 6);
    APP_ERROR_CHECK(err_code);
    
    // Reverse byte order
    uint8_t *data;
    data = (uint8_t*)accel_values;
    for(uint8_t i = 0; i<6; i++)
    {
        *data = raw_values[5-i];
        data++;
    }
    return err_code;
}


uint32_t mpu9150_read_gyro(gyro_values_t * gyro_values)
{
    uint32_t err_code;
    uint8_t raw_values[6];
    err_code = twi_read_burst(MPU9150_REG_GYRO_XOUT_H, raw_values, 6);
    APP_ERROR_CHECK(err_code);
    
    // Reverse byte order
    uint8_t *data;
    data = (uint8_t*)gyro_values;
    for(uint8_t i = 0; i<6; i++)
    {
        *data = raw_values[5-i];
        data++;
    }
    return err_code;
}

uint32_t mpu9150_read_magn(magn_values_t * magn_values)
{
//    uint32_t err_code;
//    err_code = twi_read_burst(MPU9150_REG_MAGN_XOUT_L, (uint8_t *)magn_values, 6);
//    APP_ERROR_CHECK(err_code);
//    
//    return err_code;
    uint32_t err_code;
    uint8_t raw_values[6];
    err_code = twi_read_burst(MPU9150_REG_GYRO_XOUT_H, raw_values, 6);
    APP_ERROR_CHECK(err_code);
    
    // Reverse byte order
    uint8_t *data;
    data = (uint8_t*)magn_values;
    for(uint8_t i = 0; i<6; i++)
    {
        *data = raw_values[i];
        data++;
    }
    return err_code;
}


uint32_t mpu9150_read_temp(temp_value_t * temperature)
{
    uint32_t err_code;
    uint8_t raw_values[2];
    err_code = twi_read_burst(MPU9150_REG_TEMP_OUT_H, raw_values, 2);
    APP_ERROR_CHECK(err_code);  
    
    *temperature = (temp_value_t)(raw_values[0] << 8) + raw_values[1];
    
    return err_code;    
}

uint32_t mpu9150_accel_sensitivity_get(uint16_t * lsb_pr_mg)
{
    uint32_t err_code;
    accel_config_t raw_values;
    err_code = twi_read_burst(MPU9150_REG_ACCEL_CONFIG, (uint8_t*)&raw_values, sizeof(raw_values));
    APP_ERROR_CHECK(err_code);  
    switch(raw_values.afs_sel)
    {
        case MPU_ACCEL_RANGE_2G:
            *lsb_pr_mg = 16384;
            break;
        case MPU_ACCEL_RANGE_4G:
            *lsb_pr_mg = 8192;
            break;
        case MPU_ACCEL_RANGE_8G:
            *lsb_pr_mg = 4096;
            break;
        case MPU_ACCEL_RANGE_16G:
            *lsb_pr_mg = 2048;
            break;
        default:
            *lsb_pr_mg = 0xFFFF;
            return MPU9150_ERR_UNKNOWN;
    }

    
    return err_code; 
}    


uint32_t mpu9150_gyro_sensitivity_get(float * lsb_pr_dps)
{
    uint32_t err_code;
    gyro_config_t raw_values;
    err_code = twi_read_burst(MPU9150_REG_GYRO_CONFIG, (uint8_t*)&raw_values, sizeof(raw_values));
    APP_ERROR_CHECK(err_code);  
    switch(raw_values.fs_sel)
    {
        case MPU_GYRO_RANGE_250DPS:
            *lsb_pr_dps = 131;
            break;
        case MPU_GYRO_RANGE_500DPS:
            *lsb_pr_dps = 65.5;
            break;
        case MPU_GYRO_RANGE_1000DPS:
            *lsb_pr_dps = 32.8;
            break;
        case MPU_GYRO_RANGE_2000DPS:
            *lsb_pr_dps = 16.4;
            break;
        default:
            *lsb_pr_dps = 0xFFFFFFFF;
            return MPU9150_ERR_UNKNOWN;
    }   
    return err_code; 
}

uint32_t mpu9150_self_test(accel_values_t * test_results)
{
    uint32_t err_code;
    // Read accel config register to not overwrite settings
    uint8_t config_data;
    err_code = twi_read_burst(MPU9150_REG_ACCEL_CONFIG, &config_data, sizeof(config_data));
    APP_ERROR_CHECK(err_code);      
    
    // Set self test start
    uint8_t test_data = 0xE0 | config_data; // Set self test starter bits
    err_code = twi_write_burst(MPU9150_REG_ACCEL_CONFIG, &test_data, sizeof(test_data));
    APP_ERROR_CHECK(err_code);   

    // Read self test values
    nrf_delay_ms(50);
    mpu9150_read_accel(test_results);
    
    // Self test stop
    err_code = twi_write_burst(MPU9150_REG_ACCEL_CONFIG, &config_data, sizeof(test_data));
    APP_ERROR_CHECK(err_code);  
    
    return 0;
}
/**
  @}
*/
