/* 
 *
 */

#ifndef MPU9150_H
#define MPU9150_H


#include <stdbool.h>
#include <stdint.h>
#include "nrf_drv_twi.h"

#define MPU9150_ADDRESS     0x68 

#define MPU9150_MPU9150_BASE_NUM    0x4000
#define MPU9150_SUCCESS             (MPU9150_MPU9150_BASE_NUM + 0)
#define MPU9150_ERR_TWI_TIMEOUT     (MPU9150_MPU9150_BASE_NUM + 1)
#define MPU9150_ERR_UNKNOWN         (MPU9150_MPU9150_BASE_NUM + 2)

// Accelerometer range settings
enum Ascale {
  MPU_ACCEL_RANGE_2G = 0,
  MPU_ACCEL_RANGE_4G,
  MPU_ACCEL_RANGE_8G,
  MPU_ACCEL_RANGE_16G
};

// Gyroscope range settings
enum Gscale {
  MPU_GYRO_RANGE_250DPS = 0,
  MPU_GYRO_RANGE_500DPS,
  MPU_GYRO_RANGE_1000DPS,
  MPU_GYRO_RANGE_2000DPS
};

// Acceleration high pass filter settings
enum Accel_HPF {
    MPU9150_ACCEL_HPF_RESET = 0,
    MPU9150_ACCEL_HPF_CUT_OFF_5HZ,
    MPU9150_ACCEL_HPF_CUT_OFF_2_5HZ,
    MPU9150_ACCEL_HPF_CUT_OFF_1_25HZ,
    MPU9150_ACCEL_HPF_CUT_OFF_0_63HZ,
    MPU9150_ACCEL_HPF_HOLD = 7
};

// Struct holding accelerometer values
typedef struct
{
    int16_t z;
    int16_t y;
    int16_t x;
}accel_values_t;

// Struct holding gyro values
typedef struct
{
    int16_t z;
    int16_t y;
    int16_t x;
}gyro_values_t;

// Struct holding magnetometer values
typedef struct
{
    int16_t z;
    int16_t y;
    int16_t x;
}magn_values_t;

// Struct holding temperature values
typedef int16_t temp_value_t;


// Struct holding configure values for the external Frame Synchronization (FSYNC) pin sampling and the Digital
// Low Pass Filter (DLPF) setting for both the gyroscopes and accelerometers.
typedef struct
{
    uint8_t dlpf_cfg     :3;
    uint8_t ext_sync_set :3;
}sync_dlpf_config_t;

// Struct holding Gyro configuration values
typedef struct
{
    uint8_t                 :3;
    uint8_t fs_sel          :2;
}gyro_config_t;

// Struct holding accelerometer configuration values
typedef struct accel_config_t
{
    uint8_t accel_hpf       :3;
    uint8_t afs_sel         :2;
    uint8_t                 :3; // Self test registers
}accel_config_t;

// Struct holding basic configuration values for register 0x19 to 0x22
typedef struct
{
    uint8_t             smplrt_div;
    sync_dlpf_config_t  sync_dlpf_gonfig;
    gyro_config_t       gyro_config; 
    accel_config_t      accel_config;
}mpu9150_config_t;

/**@brief MPU9150 instance default configuration. */
#define MPU9150_DEFAULT_CONFIG()                          \
    {                                                     \
        .smplrt_div                     = 7,                          \
        .sync_dlpf_gonfig.dlpf_cfg      = 0,                          \
        .sync_dlpf_gonfig.ext_sync_set  = 0,                          \
        .gyro_config.fs_sel             = MPU_GYRO_RANGE_2000DPS,     \
        .accel_config.afs_sel           = MPU_ACCEL_RANGE_16G,        \
        .accel_config.accel_hpf         = MPU9150_ACCEL_HPF_CUT_OFF_5HZ,        \
    }
    

// Event handler handling det events generated by the TWI    
void mpu9150_on_twi_event(nrf_drv_twi_evt_t *evt);
    
// Initiating the MPU reseting sensor valies and powering up the MPU
uint32_t mpu9150_init(nrf_drv_twi_t const * const p_instance);
    
// Configure the MPU
uint32_t mpu9150_config(mpu9150_config_t * config);
    
// Write a single or a series of consecutive registers
uint32_t mpu9150_write_registers(uint8_t reg, uint8_t const * p_data, uint32_t length);
    
// Read a single or a series of consecutive registers
uint32_t mpu9150_read_registers(uint8_t reg, uint8_t * p_data, uint32_t length);

// Read accel sensor values    
uint32_t mpu9150_read_accel(accel_values_t * accel_values);

// Read gyro sensor values    
uint32_t mpu9150_read_gyro(gyro_values_t * gyro_values);

// Read gyro magnetometer values    
uint32_t mpu9150_read_magn(magn_values_t * magn_values);

// Read temperature sensor values    
uint32_t mpu9150_read_temp(temp_value_t * temp_values);

// Get accelerometer lsb sensitivity
uint32_t mpu9150_accel_sensitivity_get(uint16_t * sensitivity);

// Get gyro lsb sensitivity
uint32_t mpu9150_gyro_sensitivity_get(float * lsb_pr_dps);

uint32_t mpu9150_self_test(accel_values_t * test_results);

#endif /* MPU9150_H */

/**
  @}
*/
